@@ -1,4 +1,7 @@
 # -*- coding: iso-8859-15 -*-
+# This program is public domain
+# Author: Paul Kienzle
+# Based on spreadsheet by Les Slaback (1998).
 r"""
 Calculate expected neutron activation from time spent in beam line.
 
@@ -36,7 +39,7 @@ Example::
     >>> sample.calculate_activation(env, exposure=10, rest_times=[0, 1, 24, 360])
     >>> sample.show_table()
                                           ----------------- activity (uCi) ------------------
-    isotope  product  reaction T1/2 (hrs)        0 hrs        1 hrs       24 hrs      360 hrs
+    isotope  product  reaction  half-life        0 hrs        1 hrs       24 hrs      360 hrs
     -------- -------- -------- ---------- ------------ ------------ ------------ ------------
     Co-59    Co-60         act    5.272 y     0.000496     0.000496    0.0004958    0.0004933
     Co-59    Co-60m+       act     10.5 m        1.664       0.0317          ---          ---
@@ -50,8 +53,9 @@ Example::
 The default rest times used above show the sample activity at the end of neutron
 activation and after 1 hour, 1 day, and 15 days.
 
-Activation cross sections are mostly from IAEA-273\ [#IAEA1987]. See activation.dat in the
-package directory for details on individual isotopes.
+The neutron activation table, *activation.dat*,\ [#Shleien1998]_ contains
+details about the individual isotopes, with interaction cross sections taken
+from from IAEA-273\ [#IAEA1987]_.
 
 Activation can be run from the command line using::
 
@@ -59,10 +63,14 @@ Activation can be run from the command line using::
 
 where FORMULA is the chemical formula for the material.
 
-..[#IAEA1987] IAEA (1987)
-Handbook on Nuclear Activation Data.
-TR 273 (International Atomic Energy Agency, Vienna, Austria, 1987).
-https://www-nds.iaea.org/publications/tecdocs/sti%252Fdoc%252F10%252F0273/
+.. [#Shleien1998] Shleien, B., Slaback, L.A., Birky, B.K., 1998.
+   Handbook of health physics and radiological health.
+   Williams & Wilkins, Baltimore.
+
+.. [#IAEA1987] IAEA (1987)
+   Handbook on Nuclear Activation Data.
+   TR 273 (International Atomic Energy Agency, Vienna, Austria, 1987).
+   http://cds.cern.ch/record/111089/files/IAEA-TR-273.pdf
 """
 
 from __future__ import division, print_function
@@ -106,8 +114,9 @@ class Sample(object):
 
     *formula* : chemical formula
 
-        Chemical formula.  Any format accepted by :func:`periodictable.formula` can be
-        used, including formula string.
+        Chemical formula.  Any format accepted by
+        :func:`.formulas.formula` can be used, including
+        formula string.
 
     *mass* : float | g
 
@@ -142,7 +151,7 @@ class Sample(object):
 
         *abundance* is a function that returns the relative abundance of an isotope.  By
         default it uses :func:`NIST2001_isotopic_abundance`, and there is the alternative
-        :func:`IAEA273_isotopic_abundance`.
+        :func:`IAEA1987_isotopic_abundance`.
         """
         self.activity = {}
         self.environment = environment
@@ -178,9 +187,8 @@ class Sample(object):
         # Return target time, or 0 if target time is negative
         if f(0) < target:
             return 0
-        else:
-            t, ft = find_root(0, f, df)
-            return t
+        t, ft = find_root(0, f, df)
+        return t
 
     def _accumulate(self, activity):
         for el, activity_el in activity.items():
@@ -221,7 +229,7 @@ class Sample(object):
 
         # Print the table header, with an overbar covering the various rest times
         # Print a dashed separator above and below each column
-        header = ["isotope", "product", "reaction", "T1/2 (hrs)"] \
+        header = ["isotope", "product", "reaction", "half-life"] \
                  + ["%g hrs"%vi for vi in self.rest_times]
         separator = ["-"*8, "-"*8, "-"*8, "-"*10] + ["-"*12]*len(self.rest_times)
         cformat = "%-8s %-8s %8s %10s " + " ".join(["%12s"]*len(self.rest_times))
@@ -359,7 +367,7 @@ def activity(isotope, mass, env, exposure, rest_times):
         #    ai.daughter
         # Column G: Half-life
         #    ai.Thalf_str
-        # Column H: initial efective cross-section (b)
+        # Column H: initial effective cross-section (b)
         initialXS = ai.thermalXS + env.epithermal_reduction_factor*ai.resonance
         # Column I: reaction
         #    ai.reaction
@@ -402,9 +410,12 @@ def activity(isotope, mass, env, exposure, rest_times):
             product_2n = lam if ai.reaction == '2n' else 0
             # Column T: activity if 2n mode
             activity = root*lam*(parent_activity-parent_lam)*(
-                exp(-lam_2n*exposure) / ((parent_activity-lam_2n)*(product_2n-lam_2n))
-                + exp(-parent_activity*exposure) / ((lam_2n-parent_activity)*(product_2n-parent_activity))
-                + exp(-product_2n*exposure) / ((lam_2n-product_2n)*(parent_activity-product_2n))
+                (exp(-lam_2n*exposure)
+                 / ((parent_activity-lam_2n)*(product_2n-lam_2n)))
+                + (exp(-parent_activity*exposure)
+                   / ((lam_2n-parent_activity)*(product_2n-parent_activity)))
+                + (exp(-product_2n*exposure)
+                   / ((lam_2n-product_2n)*(parent_activity-product_2n)))
                 )
             #print("N", parent_lam, "P", effectiveXS, "Q", lam_2n,
             #      "R", parent_activity, "S", product_2n, "T", activity)
@@ -459,7 +470,6 @@ def init(table, reload=False):
                 del el[iso].neutron_activation
 
     path = os.path.join(core.get_data_path('.'), 'activation.dat')
-    lastA = 0
     for row in open(path, 'r'):
         columns = row.split('\t')
         if columns[0].strip() in ('', 'xx'):
